#include <tcl.h>
#include <stdio.h>
#include <stdlib.h>

char *crypt(const char *key, const char *salt);
int time(char *tm);

static unsigned char itoa64[] =         /* 0 ... 63 => ascii - 64 */
        "./0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz";

static int
Passwd (ClientData clientData, Tcl_Interp *interp, int argc, char *argv[])
{
    char salt[3], buf[32];
    long v;

    if (argc == 2) {
        /* Encrypt a password. */
        srand((int)time((char *) NULL));
        v = rand();
        salt[0] = itoa64[v & 0x3f];
        v >>= 6;
        salt[1] = itoa64[v & 0x3f];
        sprintf(buf, "%s",crypt(argv[1], salt));
	Tcl_AppendResult (interp, buf, (char *) NULL);
	return TCL_OK;
    }
    if (argc == 3) {
        /* Compare clear_pw password to encrypted_pw version. */
        interp->result = ((strcmp(argv[2],crypt(argv[1],argv[2])))? "0":"1");
	return TCL_OK;
    }
    Tcl_AppendResult (interp, "wrong # args: ", argv[0],
	    " clear_pw ?encrypted_pw?", (char *) NULL);
    return TCL_ERROR;
}

int
Tclpasswd_Init(Tcl_Interp *interp)
{
#ifdef USE_TCL_STUBS
	if (Tcl_InitStubs(interp, TCL_VERSION, 0) == NULL) {
	    return TCL_ERROR;
	}
#else
        if (Tcl_PkgRequire(interp, "Tcl", TCL_VERSION, 0) == NULL) {
            return TCL_ERROR;
        }
#endif
        if (Tcl_PkgProvide(interp, "Tclpasswd", VERSION) != TCL_OK) {
            return TCL_ERROR;
        }
        Tcl_CreateCommand (interp, "passwd", Passwd,
                (ClientData)NULL, (Tcl_CmdDeleteProc *)NULL);

        return TCL_OK;
}
